@@ -52,6 +52,7 @@ from repo_helper._docs_shields import (
 		make_docs_maintained_shield,
 		make_docs_pre_commit_ci_shield,
 		make_docs_pre_commit_shield,
+		make_docs_pypi_downloads_shield,
 		make_docs_pypi_version_shield,
 		make_docs_python_implementations_shield,
 		make_docs_python_versions_shield,
@@ -76,6 +77,7 @@ from repo_helper.shields import (
 		make_maintained_shield,
 		make_pre_commit_ci_shield,
 		make_pre_commit_shield,
+		make_pypi_downloads_shield,
 		make_pypi_version_shield,
 		make_python_implementations_shield,
 		make_python_versions_shield,
@@ -311,12 +313,22 @@ class ShieldsBlock:
 	:param platforms: List of supported platforms.
 	:param pre_commit: Whether to show a shield for pre-commit
 	:param on_pypi:
+	:param primary_conda_channel: The Conda channel the package can be downloaded from.
 
 	.. versionadded:: 2020.12.11
 	"""
 
 	#: This list controls which sections are included, and their order.
-	sections = ("Docs", "Tests", "PyPI", "Anaconda", "Activity", "Docker", "Other")
+	sections = (
+			"Docs",
+			"Tests",
+			"PyPI",
+			"Anaconda",
+			"Activity",
+			"QA",
+			"Docker",
+			"Other",
+			)
 
 	#: This list controls which substitutions are included, and their order.
 	substitutions = (
@@ -325,6 +337,8 @@ class ShieldsBlock:
 			"actions_linux",
 			"actions_windows",
 			"actions_macos",
+			"actions_flake8",
+			"actions_mypy",
 			"requires",
 			"coveralls",
 			"codefactor",
@@ -339,6 +353,7 @@ class ShieldsBlock:
 			"commits-since",
 			"commits-latest",
 			"maintained",
+			"pypi-downloads",
 			"docker_build",
 			"docker_automated",
 			"docker_size",
@@ -363,6 +378,7 @@ class ShieldsBlock:
 			platforms: Optional[Iterable[str]] = None,
 			pre_commit: bool = False,
 			on_pypi: bool = True,
+			primary_conda_channel: Optional[str] = None,
 			):
 
 		if unique_name and not unique_name.startswith('_'):
@@ -382,6 +398,7 @@ class ShieldsBlock:
 		self.platforms: Iterable[str] = set(platforms or ())
 		self.pre_commit: bool = pre_commit
 		self.on_pypi: bool = on_pypi
+		self.primary_conda_channel: str = primary_conda_channel or self.username
 
 		self.set_readme_mode()
 
@@ -412,6 +429,7 @@ class ShieldsBlock:
 		self.make_requires_shield = make_requires_shield
 		self.make_rtfd_shield = make_rtfd_shield
 		self.make_wheel_shield = make_wheel_shield
+		self.make_pypi_downloads_shield = make_pypi_downloads_shield
 
 	def set_docs_mode(self) -> None:
 		"""
@@ -440,6 +458,7 @@ class ShieldsBlock:
 		self.make_requires_shield = make_docs_requires_shield
 		self.make_rtfd_shield = make_docs_rtfd_shield
 		self.make_wheel_shield = make_docs_wheel_shield
+		self.make_pypi_downloads_shield = make_docs_pypi_downloads_shield
 
 	def make(self) -> str:
 		"""
@@ -474,11 +493,9 @@ class ShieldsBlock:
 		substitutions["license"] = self.make_license_shield(repo_name, username)
 		substitutions["language"] = self.make_language_shield(repo_name, username)
 
-		sections["QA"] = ["codefactor", "actions_macos"]
+		sections["QA"] = ["codefactor", "actions_flake8", "actions_mypy"]
 		substitutions["codefactor"] = self.make_codefactor_shield(repo_name, username)
-		sections["QA"].append("actions_flake8")
 		substitutions["actions_flake8"] = self.make_actions_shield(repo_name, username, "Flake8", "Flake8 Status")
-		sections["QA"].append("actions_mypy")
 		substitutions["actions_mypy"] = self.make_actions_shield(repo_name, username, "mypy", "mypy status")
 
 		if self.docs:
@@ -517,16 +534,10 @@ class ShieldsBlock:
 			sections["Tests"].append("coveralls")
 			substitutions["coveralls"] = self.make_coveralls_shield(repo_name, username)
 
-		sections["Tests"].append("codefactor")
-
 		if self.pre_commit:
 			sections["QA"].append("pre_commit_ci")
-			sections["Tests"].append("pre_commit_ci")
 			substitutions["pre_commit_ci"] = self.make_pre_commit_ci_shield(repo_name, username)
 
-			sections["Other"].append("pre_commit")
-			substitutions["pre_commit"] = self.make_pre_commit_shield()
-
 		if self.on_pypi:
 			sections["PyPI"] = ["pypi-version", "supported-versions", "supported-implementations", "wheel"]
 			substitutions["pypi-version"] = self.make_pypi_version_shield(pypi_name)
@@ -534,10 +545,15 @@ class ShieldsBlock:
 			substitutions["supported-implementations"] = self.make_python_implementations_shield(pypi_name)
 			substitutions["wheel"] = self.make_wheel_shield(pypi_name)
 
+			sections["Activity"].append("pypi-downloads")
+			substitutions["pypi-downloads"] = self.make_pypi_downloads_shield(pypi_name)
+
 		if self.conda:
 			sections["Anaconda"] = ["conda-version", "conda-platform"]
-			substitutions["conda-version"] = self.make_conda_version_shield(pypi_name, username)
-			substitutions["conda-platform"] = self.make_conda_platform_shield(pypi_name, username)
+			substitutions["conda-version"] = self.make_conda_version_shield(pypi_name, self.primary_conda_channel)
+			substitutions["conda-platform"] = self.make_conda_platform_shield(
+					pypi_name, self.primary_conda_channel
+					)
 
 		if self.docker_shields:
 			docker_name = self.docker_name
@@ -547,7 +563,7 @@ class ShieldsBlock:
 			substitutions["docker_size"] = self.make_docker_size_shield(docker_name, username)
 
 		for section in self.sections:
-			if section not in sections:
+			if section not in sections or not sections[section]:
 				continue
 
 			images = DelimitedList([f"|{name}{self.unique_name}|" for name in sections[section]])
