@@ -5,7 +5,7 @@
 :pep:`517` build backend.
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU Lesser General Public License as published by
@@ -33,10 +33,8 @@ import shutil
 import sys
 import tarfile
 import tempfile
-from base64 import urlsafe_b64encode
 from datetime import datetime
 from email.message import EmailMessage
-from hashlib import sha256
 from io import StringIO
 from subprocess import PIPE, Popen
 from textwrap import dedent, indent
@@ -91,7 +89,7 @@ class Builder:
 		self.tag = "py3-none-any"
 
 		#: repo_helper's configuration dictionary.
-		self.config = parse_yaml(self.repo_dir)
+		self.config = parse_yaml(self.repo_dir, allow_unknown_keys=True)
 
 		self.config["version"] = str(Version(self.config["version"]))
 
@@ -154,8 +152,7 @@ class Builder:
 		for py_pattern in {"**/*.py", "**/*.pyi", "**/*.pyx", "**/py.typed"}:
 			for py_file in pkgdir.rglob(py_pattern):
 				if "__pycache__" not in py_file.parts:
-					# ref: https://github.com/python/typeshed/issues/4746
-					yield py_file  # type: ignore
+					yield py_file
 
 	def copy_source(self) -> None:
 		"""
@@ -367,7 +364,7 @@ class Builder:
 		for classifier in self.config["classifiers"]:
 			metadata["Classifier"] = classifier
 
-		metadata["Requires-Python"] = str(Specifier(f">={self.config['min_py_version']}"))
+		metadata["Requires-Python"] = str(Specifier(f">={self.config['requires_python']}"))
 		metadata["Description-Content-Type"] = "text/x-rst"
 
 		for requirement in sorted(combine_requirements(read_requirements(self.repo_dir / "requirements.txt")[0])):
