@@ -32,16 +32,16 @@ import os.path
 import pathlib
 import shutil
 import warnings
-from textwrap import dedent
-from typing import Dict, List, Sequence, Union
+from contextlib import suppress
+from typing import Dict, List, Mapping, MutableMapping, Tuple, Union
 
 # 3rd party
-import css_parser  # type: ignore
+import dict2css
 import jinja2
 import ruamel.yaml as yaml
-from css_parser import css
 from domdf_python_tools.compat import importlib_resources
 from domdf_python_tools.paths import PathPlus
+from domdf_python_tools.stringlist import StringList
 from domdf_python_tools.typing import PathLike
 from domdf_python_tools.utils import enquote_value
 from shippinglabel import normalize
@@ -75,14 +75,12 @@ __all__ = [
 		"make_rtfd",
 		"make_docutils_conf",
 		"make_conf",
-		"StyleSheet",
 		"make_alabaster_theming",
 		"make_readthedocs_theming",
 		"copy_docs_styling",
 		"rewrite_docs_index",
 		"make_404_page",
 		"make_docs_source_rst",
-		"make_style",
 		]
 
 # Disable logging from cssutils
@@ -103,7 +101,6 @@ class DocRequirementsManager(RequirementsManager):
 			# ComparableRequirement("sphinx_autodoc_typehints>=1.11.0"),
 			ComparableRequirement("sphinx-copybutton>=0.2.12"),
 			ComparableRequirement("sphinx-prompt>=1.1.0"),
-			ComparableRequirement("sphinx>=3.0.3"),
 			}
 
 	def __init__(self, repo_path: PathLike, templates: jinja2.Environment):
@@ -115,7 +112,7 @@ class DocRequirementsManager(RequirementsManager):
 	theme_versions = {
 			"alabaster": ">=0.7.12",
 			"sphinx-rtd-theme": "<0.5",
-			"domdf-sphinx-theme": ">=0.1.0",
+			"domdf-sphinx-theme": ">=0.3.0",
 			"repo-helper-sphinx_theme": ">=0.0.2",
 			"furo": ">=2020.11.19b18",
 			}
@@ -132,10 +129,11 @@ class DocRequirementsManager(RequirementsManager):
 		# Mapping of pypi_name to version specifier
 		my_sphinx_extensions = {
 				"extras-require": ">=0.2.0",
-				"seed-intersphinx-mapping": ">=0.1.1",
-				"default-values": ">=0.2.0",
-				"toctree-plus": ">=0.0.4",
-				"sphinx-toolbox": ">=1.8.2",
+				"seed-intersphinx-mapping": ">=0.3.1",
+				"default-values": ">=0.4.2",
+				"toctree-plus": ">=0.1.0",
+				"sphinx-toolbox": ">=1.8.3",
+				"sphinx-debuginfo": ">=0.1.0",
 				}
 
 		for name, specifier in my_sphinx_extensions.items():
@@ -143,7 +141,14 @@ class DocRequirementsManager(RequirementsManager):
 				self.target_requirements.add(ComparableRequirement(f"{name}{specifier}"))
 
 	def merge_requirements(self) -> List[str]:
-		current_requirements, comments, invalid_lines = read_requirements(self.req_file, include_invalid=True)
+		current_requirements_, comments, invalid_lines = read_requirements(self.req_file, include_invalid=True)
+
+		current_requirements = list(current_requirements_)
+
+		if "sphinx" not in current_requirements:
+			current_requirements.append(ComparableRequirement("sphinx>=3.0.3,<3.4.0"))
+		else:
+			current_requirements.append(ComparableRequirement("sphinx>=3.0.3"))
 
 		for line in invalid_lines:
 			if line.startswith("git+"):
@@ -204,24 +209,25 @@ def make_rtfd(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[st
 			*templates.globals["additional_requirements_files"],
 			]
 
-	python_config = {"version": 3.8, "install": [{"requirements": r} for r in install_requirements]}
+	install_config: List[Dict] = [{"requirements": r} for r in install_requirements]
+	install_config.append({"method": "pip", "path": '.'})
+
+	python_config = {"version": 3.8, "install": install_config}
 
 	# Formats: Optionally build your docs in additional formats such as PDF and ePub
 	config = {"version": 2, "sphinx": sphinx_config, "formats": "all", "python": python_config}
 
-	class Dumper(yaml.RoundTripDumper):
+	dumper = yaml.YAML()
+	dumper.indent(mapping=2, sequence=3, offset=1)
 
-		@functools.wraps(yaml.RoundTripDumper.__init__)
-		def __init__(self, *args, **kwargs):
-			super().__init__(*args, **kwargs)
-			self.sequence_dash_offset = 1
-			self.best_width = 4086
+	yaml_buf = yaml.StringIO()
+	dumper.dump(config, yaml_buf)
 
 	file.write_lines([
 			f"# {templates.globals['managed_message']}",
 			"# Read the Docs configuration file",
 			"---",
-			yaml.round_trip_dump(config, default_flow_style=False, Dumper=Dumper),  # type: ignore
+			yaml_buf.getvalue()
 			])
 
 	return [file.relative_to(repo_path).as_posix()]
@@ -281,62 +287,65 @@ def make_conf(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[st
 	repo_name = templates.globals["repo_name"]
 
 	if templates.globals["sphinx_html_theme"] in {"sphinx-rtd-theme", "domdf-sphinx-theme"}:
-		for key, val in {
-			"display_github": True,  # Integrate GitHub
-			"github_user": username,  # Username
-			"github_repo": repo_name,  # Repo name
-			"github_version": "master",  # Version
-			"conf_py_path": f"/{templates.globals['docs_dir']}/",  # Path in the checkout to the docs root
-			}.items():
+		style = {
+				"display_github": True,  # Integrate GitHub
+				"github_user": username,  # Username
+				"github_repo": repo_name,  # Repo name
+				"github_version": "master",  # Version
+				"conf_py_path": f"/{templates.globals['docs_dir']}/",  # Path in the checkout to the docs root
+				}
+
+		for key, val in style.items():
 			if key not in templates.globals["html_context"]:
 				templates.globals["html_context"][key] = val
 
-		for key, val in {
-			# 'logo': 'logo.png',
-			"logo_only": False,  # True will show just the logo
-			}.items():
+		options = {
+				# 'logo': 'logo.png',
+				"logo_only": False,  # True will show just the logo
+				}
+
+		for key, val in options.items():
 			if key not in templates.globals["html_theme_options"]:
 				templates.globals["html_theme_options"][key] = val
 
 	elif templates.globals["sphinx_html_theme"] in {"alabaster", "repo-helper-sphinx-theme"}:
 		# See https://github.com/bitprophet/alabaster/blob/master/alabaster/theme.conf
 		# and https://alabaster.readthedocs.io/en/latest/customization.html
-		for key, val in {
-			# 'logo': 'logo.png',
-			"page_width": "1200px",
-			"logo_name": "true",
-			"github_user": username,  # Username
-			"github_repo": repo_name,  # Repo name
-			"description": templates.globals["short_desc"],
-			"github_banner": "true",
-			"github_type": "star",
-			# "travis_button": "true",
-			"badge_branch": "master",
-			"fixed_sidebar": "true",
-			}.items():
+
+		style = {
+				# 'logo': 'logo.png',
+				"page_width": "1200px",
+				"logo_name": "true",
+				"github_user": username,  # Username
+				"github_repo": repo_name,  # Repo name
+				"description": templates.globals["short_desc"],
+				"github_banner": "true",
+				"github_type": "star",
+				"badge_branch": "master",
+				"fixed_sidebar": "true",
+				}
+
+		for key, val in style.items():
 			if key not in templates.globals["html_theme_options"]:
 				templates.globals["html_theme_options"][key] = val
 
 	elif templates.globals["sphinx_html_theme"] in {"furo"}:
 		# See https://github.com/bitprophet/alabaster/blob/master/alabaster/theme.conf
 		# and https://alabaster.readthedocs.io/en/latest/customization.html
-		for key, val in {
-			"light_css_variables": {
-			"toc-title-font-size": "12pt",
-			"toc-font-size": "12pt",
-			"admonition-font-size": "12pt",
-			},
-			"dark_css_variables": {
-			"toc-title-font-size": "12pt",
-			"toc-font-size": "12pt",
-			"admonition-font-size": "12pt",
-			},
-			}.items():
+
+		style = {
+				"toc-title-font-size": "12pt",
+				"toc-font-size": "12pt",
+				"admonition-font-size": "12pt",
+				}
+
+		for key in ["light_css_variables", "dark_css_variables"]:
 			if key not in templates.globals["html_theme_options"]:
-				templates.globals["html_theme_options"][key] = val
+				templates.globals["html_theme_options"][key] = style
 			else:
 				templates.globals["html_theme_options"][key] = {
-						**val, **templates.globals["html_theme_options"][key]
+						**style,
+						**templates.globals["html_theme_options"][key],
 						}
 
 	sphinx_extensions = [
@@ -354,8 +363,10 @@ def make_conf(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[st
 			"sphinx_copybutton",
 			"sphinxcontrib.default_values",
 			"sphinxcontrib.toctree_plus",
-			"seed_intersphinx_mapping",  # "sphinx.ext.autosectionlabel",
+			"sphinx_debuginfo",
+			"seed_intersphinx_mapping",
 			]
+	# "sphinx.ext.autosectionlabel",
 	# "sphinx_gitstamp",
 
 	sphinx_extensions.extend(templates.globals["extra_sphinx_extensions"])
@@ -375,22 +386,6 @@ def make_conf(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[st
 	return [file.relative_to(repo_path).as_posix()]
 
 
-class StyleSheet(css.CSSStyleSheet):
-	"""
-	Represents a CSS style sheet.
-	"""
-
-	def add_style(self, selector: str, styles: Dict[str, Union[Sequence, str, int, None]]) -> None:
-		"""
-		Add a style to the stylesheet.
-
-		:param selector:
-		:param styles:
-		"""
-
-		self.add(make_style(selector, styles))
-
-
 def make_alabaster_theming() -> str:
 	"""
 	Make the custom stylesheet for the alabaster Sphinx theme.
@@ -398,131 +393,84 @@ def make_alabaster_theming() -> str:
 	:return: The custom stylesheet.
 	"""
 
-	sheet = StyleSheet()
-
-	# Reset CSS Parser to defaults
-	css_parser.ser.prefs.useDefaults()
-
-	# Formatting preferences
-	css_parser.ser.prefs.omitLastSemicolon = False
-	css_parser.ser.prefs.indentClosingBrace = False
-	css_parser.ser.prefs.indent = '\t'
-
-	# Helpers
-	important = "important"
-
-	def px(val: Union[int, float, str]) -> str:
-		return f"{val}px"
-
 	# Common options
 	solid_border = {"border-style": "solid"}
-	docs_bottom_margin = {"margin-bottom": (px(17), important)}
+	docs_bottom_margin = {"margin-bottom": (dict2css.px(17), dict2css.IMPORTANT)}
 
-	sheet.add_style("li p:last-child", {"margin-bottom": px(12)})
+	BorderMappingType = MutableMapping[str, Union[str, Tuple[str, str]]]
 
-	# Smooth scrolling between sections
-	sheet.add_style("html", {"scroll-behavior": "smooth"})
+	object_border: BorderMappingType = {
+			"padding": "3px 3px 3px 5px",
+			**docs_bottom_margin,
+			**solid_border,
+			}
 
-	# Border around classes
-	sheet.add_style(
-			"dl.class",
-			{
-					"padding": "3px 3px 3px 5px",
-					"margin-top": ("7px", important),
-					**docs_bottom_margin,
-					"border-color": "rgba(240, 128, 128, 0.5)",
-					**solid_border,
-					}
-			)
+	class_border: BorderMappingType = {
+			**object_border,
+			"margin-top": ("7px", dict2css.IMPORTANT),
+			"border-color": "rgba(240, 128, 128, 0.5)",
+			}
 
-	# Border around functions
-	sheet.add_style(
-			"dl.function",
-			{
-					"padding": "3px 3px 3px 5px",
-					"margin-top": ("7px", important),
-					**docs_bottom_margin,
-					"border-color": "lightskyblue",
-					**solid_border,
-					}
-			)
+	function_border: BorderMappingType = {
+			**object_border,
+			"margin-top": ("7px", dict2css.IMPORTANT),
+			"border-color": "lightskyblue",
+			}
 
-	sheet.add_style("dl.function dt", {"margin-bottom": (px(10), important)})
+	attribute_border: BorderMappingType = {**object_border, "border-color": "rgba(119, 136, 153, 0.5)"}
+	method_border: BorderMappingType = {**object_border, "border-color": "rgba(32, 178, 170, 0.5)"}
 
-	# Border around attributes
-	sheet.add_style(
-			"dl.attribute",
-			{
-					"padding": "3px 3px 3px 5px",
-					**docs_bottom_margin,
-					"border-color": "rgba(119, 136, 153, 0.5)",
-					**solid_border
-					}
-			)
+	table_vertical_margins = {
+			"margin-bottom": (dict2css.px(20), "important"),
+			"margin-top": (dict2css.px(-15), dict2css.IMPORTANT),
+			}
 
-	# Border around Methods
-	sheet.add_style(
-			"dl.method",
-			{
-					"padding": "3px 3px 3px 5px",
-					**docs_bottom_margin,
-					"border-color": "rgba(32, 178, 170, 0.5)",
-					**solid_border
-					}
-			)
+	style: Dict[str, MutableMapping] = {
+			"li p:last-child": {"margin-bottom": dict2css.px(12)},
+			"html": {"scroll-behavior": "smooth"},  # Smooth scrolling between sections
+			"dl.class": class_border,
+			"dl.function": function_border,
+			"dl.function dt": {"margin-bottom": (dict2css.px(10), dict2css.IMPORTANT)},
+			"dl.attribute": attribute_border,
+			"dl.method": method_border,
+			"div.sphinxsidebar": {"font-size": dict2css.px(14), "line-height": "1.5"},
+			"div.sphinxsidebar h3": {"font-weight": "bold"},
+			"div.sphinxsidebar p.caption": {"font-size": dict2css.px(20)},
+			"div.sphinxsidebar div.sphinxsidebarwrapper": {"padding-right": (dict2css.px(20), dict2css.IMPORTANT)},
+			"table.longtable": table_vertical_margins,
+			}
 
-	sheet.add_style("div.sphinxsidebar", {"font-size": px(14), "line-height": "1.5"})
-	sheet.add_style("div.sphinxsidebar h3", {"font-weight": "bold"})
-	sheet.add_style("div.sphinxsidebar p.caption", {"font-size": px(20)})
-	sheet.add_style("div.sphinxsidebar div.sphinxsidebarwrapper", {"padding-right": (px(20), important)})
-
-	# Margin above and below table
-	sheet.add_style(
-			"table.longtable", {"margin-bottom": (px(20), "important"), "margin-top": (px(-15), "important")}
-			)
-
-	# The following styling from Tox"s documentation
+	# The following styling from Tox's documentation
 	# https://github.com/tox-dev/tox/blob/master/docs/_static/custom.css
 	# MIT Licensed
 
 	# Page width
-	sheet.add_style("div.document", {"width": "100%", "max-width": px(1400)})
-	sheet.add_style("div.body", {"max-width": px(1100)})
+	style["div.document"] = {"width": "100%", "max-width": dict2css.px(1400)}
+	style["div.body"] = {"max-width": dict2css.px(1100)}
 
 	# No end-of-line hyphenation
-	sheet.add_style("div.body p, ol > li, div.body td", {"hyphens": None})
+	style["div.body p, ol > li, div.body td"] = {"hyphens": None}
 
-	sheet.add_style("img, div.figure", {"margin": ('0', important)})
-	sheet.add_style("ul > li", {"text-align": "justify"})
-	sheet.add_style("ul > li > p", {"margin-bottom": '0'})
-	sheet.add_style("ol > li > p", {"margin-bottom": '0'})
-	sheet.add_style("div.body code.descclassname", {"display": None})
-	sheet.add_style(".wy-table-responsive table td", {"white-space": ("normal", important)})
-	sheet.add_style(".wy-table-responsive", {"overflow": ("visible", important)})
-	sheet.add_style("div.toctree-wrapper.compound > ul > li", {
+	style["img, div.figure"] = {"margin": ('0', dict2css.IMPORTANT)}
+	style["ul > li"] = {"text-align": "justify"}
+	style["ul > li > p"] = {"margin-bottom": '0'}
+	style["ol > li > p"] = {"margin-bottom": '0'}
+	style["div.body code.descclassname"] = {"display": None}
+	style[".wy-table-responsive table td"] = {"white-space": ("normal", dict2css.IMPORTANT)}
+	style[".wy-table-responsive"] = {"overflow": ("visible", dict2css.IMPORTANT)}
+	style["div.toctree-wrapper.compound > ul > li"] = {
 			"margin": '0',
 			"padding": '0',
-			})
+			}
 	# TODO
 	# code.docutils.literal {{
 	#     background-color: #ECF0F3;
 	#     padding: 0 1px;
 	# }}
 
-	stylesheet = sheet.cssText.decode("UTF-8").replace('}', "}\n")
+	style["@media screen and (min-width: 870px)"] = {"div.sphinxsidebar": {"width": "250px"}}
 
-	# Reset CSS Parser to defaults
-	css_parser.ser.prefs.useDefaults()
-
-	return f"""\
-{stylesheet}
-
-@media screen and (min-width: 870px) {{
-	div.sphinxsidebar {{
-		width: 250px;
-	}}
-}}
-"""
+	return dict2css.dumps(style, trailing_semicolon=True)
 
 
 def make_readthedocs_theming() -> str:
@@ -532,37 +480,13 @@ def make_readthedocs_theming() -> str:
 	:return: The custom stylesheet.
 	"""
 
-	sheet = StyleSheet()
+	style: Dict[str, Mapping] = {
+			".wy-nav-content": {"max-width": (dict2css.px(1200), dict2css.IMPORTANT)},
+			"li p:last-child": {"margin-bottom": (dict2css.px(12), dict2css.IMPORTANT)},
+			"html": {"scroll-behavior": "smooth"},
+			}
 
-	# Reset CSS Parser to defaults
-	css_parser.ser.prefs.useDefaults()
-
-	# Formatting preferences
-	css_parser.ser.prefs.omitLastSemicolon = False
-	css_parser.ser.prefs.indentClosingBrace = False
-	css_parser.ser.prefs.indent = '\t'
-
-	# Helpers
-	important = "important"
-
-	def px(val: Union[int, float, str]) -> str:
-		return f"{val}px"
-
-	# Body width
-	sheet.add_style(".wy-nav-content", {"max-width": (px(1200), important)})
-
-	# Spacing between list items
-	sheet.add_style("li p:last-child", {"margin-bottom": (px(12), important)})
-
-	# Smooth scrolling between sections
-	sheet.add_style("html", {"scroll-behavior": "smooth"})
-
-	stylesheet = sheet.cssText.decode("UTF-8").replace('}', "}\n")
-
-	# Reset CSS Parser to defaults
-	css_parser.ser.prefs.useDefaults()
-
-	return stylesheet
+	return dict2css.dumps(style, trailing_semicolon=True)
 
 
 def copy_docs_styling(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[str]:
@@ -611,21 +535,17 @@ def copy_docs_styling(repo_path: pathlib.Path, templates: jinja2.Environment) ->
 		furo_navigation.parent.maybe_make()
 		github_url = make_github_url(templates.globals["username"], templates.globals["repo_name"])
 
-		buf = [
+		buf = StringList([
 				"<!---{managed_message}--->".format_map(templates.globals),
-				dedent(
-						"""\
-<div class="sidebar-tree">
-  {{ furo_navigation_tree }}
-</div>
-
-<div class="sidebar-tree">
-  <p class="caption"><span class="caption-text">Links</span></p>
-  <ul>"""
-						)
-				]
-
-		buf.append(f'    <li class="toctree-l1"><a class="reference external" href="{github_url}">GitHub</a></li>')
+				'<div class="sidebar-tree">',
+				"  {{ furo_navigation_tree }}",
+				"</div>",
+				'',
+				'<div class="sidebar-tree">',
+				'  <p class="caption"><span class="caption-text">Links</span></p>',
+				"  <ul>",
+				f'    <li class="toctree-l1"><a class="reference external" href="{github_url}">GitHub</a></li>'
+				])
 
 		if templates.globals["on_pypi"]:
 			buf.append(
@@ -633,7 +553,8 @@ def copy_docs_styling(repo_path: pathlib.Path, templates: jinja2.Environment) ->
 					f'href="https://pypi.org/project/{templates.globals["pypi_name"]}">PyPI</a></li>'
 					)
 
-		buf.append(dedent("  </ul>\n</div>\n"))
+		buf.extend(["  </ul>", "</div>"])
+		buf.blankline(ensure_single=True)
 
 		furo_navigation.write_lines(buf)
 
@@ -649,10 +570,8 @@ def copy_docs_styling(repo_path: pathlib.Path, templates: jinja2.Environment) ->
 		layout_html.unlink(missing_ok=True)
 
 	else:
-		try:
+		with suppress(FileNotFoundError):
 			shutil.rmtree(furo_navigation.parent)
-		except FileNotFoundError:
-			pass
 
 		layout_html.write_lines([
 				f"<!--- {templates.globals['managed_message']} --->",
@@ -699,6 +618,7 @@ def rewrite_docs_index(repo_path: pathlib.Path, templates: jinja2.Environment) -
 			platforms=templates.globals["platforms"],
 			pre_commit=templates.globals["enable_pre_commit"],
 			on_pypi=templates.globals["on_pypi"],
+			primary_conda_channel=templates.globals["primary_conda_channel"],
 			)
 	sb.set_docs_mode()
 	shields_block = sb.make()
@@ -790,22 +710,3 @@ def make_docs_source_rst(repo_path: pathlib.Path, templates: jinja2.Environment)
 			docs_building_rst.relative_to(repo_path).as_posix(),
 			git_download_png.relative_to(repo_path).as_posix(),
 			]
-
-
-def make_style(selector: str, styles: Dict[str, Union[Sequence, str, int, None]]) -> css.CSSStyleRule:
-	"""
-	Create a CSS Style Rule from a dictionary.
-
-	:param selector:
-	:param styles:
-	"""
-
-	style = css.CSSStyleDeclaration()
-
-	for name, properties in styles.items():
-		if isinstance(properties, Sequence) and not isinstance(properties, str):
-			style[name] = tuple(str(x) for x in properties)
-		else:
-			style[name] = str(properties)
-
-	return css.CSSStyleRule(selectorText=selector, style=style)
