@@ -76,11 +76,8 @@ lint_warn_list = [
 		"E265",
 		"E271",
 		"E272",
-		"E301",
-		"E302",
 		"E303",
 		"E304",
-		"E305",
 		"E306",
 		"E402",
 		"E502",
@@ -211,8 +208,19 @@ lint_warn_list.extend((
 		"TYP006",  # guard typing attribute by quoting
 		))
 
+# Encodings
+lint_warn_list.extend((
+		"ENC001",  # no encoding specified for 'open'.
+		"ENC002",  # 'encoding=None' used for 'open'.
+		"ENC003",  # no encoding specified for 'open' with unknown mode.
+		"ENC004",  # 'encoding=None' used for 'open' with unknown mode.
+		))
+
 # pydocstyle
 code_only_warning = [
+		"E301",
+		"E302",
+		"E305",
 		"D100",  # Missing docstring in public module
 		"D101",  # Missing docstring in public class
 		"D102",  # Missing docstring in public method
@@ -245,7 +253,14 @@ code_only_warning = [
 
 # flake8_dunder_all
 code_only_warning.extend((
-		"DALL000 ",  # DALL000 Module lacks __all__.
+		"DALL000 ",  # Module lacks __all__.
+		))
+
+# flake8_slots
+code_only_warning.extend((
+		"SLOT000",  # Define __slots__ for subclasses of str
+		"SLOT001",  # Define __slots__ for subclasses of tuple
+		"SLOT002",  # Define __slots__ for subclasses of collections.namedtuple
 		))
 
 # flake8-pyi
