@@ -5,7 +5,7 @@
 Configuration for testing and code formatting tools.
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU Lesser General Public License as published by
@@ -29,13 +29,16 @@ import pathlib
 import posixpath
 import re
 import warnings
-from typing import Any, List
+from operator import attrgetter
+from typing import Any, Dict, List, Tuple
 
 # 3rd party
 import jinja2
+import toml
 from domdf_python_tools.paths import PathPlus
 from domdf_python_tools.stringlist import DelimitedList
 from domdf_python_tools.typing import PathLike
+from packaging.version import Version
 from shippinglabel import normalize
 from shippinglabel.requirements import (
 		ComparableRequirement,
@@ -48,26 +51,24 @@ from shippinglabel.requirements import (
 from repo_helper.configupdater2 import ConfigUpdater
 from repo_helper.files import management
 from repo_helper.files.linting import code_only_warning, lint_warn_list
-from repo_helper.utils import IniConfigurator, indent_join
+from repo_helper.utils import CustomTomlEncoder, IniConfigurator, indent_join
 
 __all__ = [
 		"make_tox",
 		"ToxConfig",
 		"make_yapf",
 		"make_isort",
+		"make_formate_toml",
 		"ensure_tests_requirements",
 		]
 
 allowed_rst_directives = ["envvar", "TODO", "extras-require"]
 standard_flake8_excludes = [
-		".git",
-		"__pycache__",
 		"old",
 		"build",
 		"dist",
 		"__pkginfo__.py",
 		"setup.py",
-		".tox",
 		"venv",
 		]
 
@@ -84,17 +85,13 @@ class ToxConfig(IniConfigurator):
 	managed_sections = [
 			"tox",
 			"envlists",
-			"gh-actions",
 			"testenv",
 			"testenv:docs",
 			"testenv:build",
 			"testenv:lint",
 			"testenv:mypy",
 			"testenv:pyup",
-			"testenv:coverage",
 			"flake8",
-			"coverage:run",
-			"coverage:report",
 			"check-wheel-contents",
 			"pytest",
 			]
@@ -104,6 +101,11 @@ class ToxConfig(IniConfigurator):
 
 		self.managed_sections = self.managed_sections[:]
 
+		if self["enable_tests"]:
+			self.managed_sections.insert(-3, "testenv:coverage")
+			self.managed_sections.insert(-2, "coverage:run")
+			self.managed_sections.insert(-2, "coverage:report")
+
 		for section_name in self["tox_unmanaged"]:
 			if section_name in self.managed_sections:
 				del self.managed_sections[self.managed_sections.index(section_name)]
@@ -155,12 +157,9 @@ class ToxConfig(IniConfigurator):
 		Compile the list of mypy dependencies.
 		"""
 
-		if self["stubs_package"]:
-			mypy_deps = ["git+https://github.com/python/mypy@c1fa1ade66a053774366d3710c380cfc8b3abbb1"]
-		else:
-			mypy_deps = [f"mypy=={self['mypy_version']}"]
+		mypy_deps = [f"mypy=={self['mypy_version']}"]
 
-		mypy_deps.append("lxml")
+		# mypy_deps.append("lxml")
 
 		if self._globals["enable_tests"]:
 			mypy_deps.append(f"-r{{toxinidir}}/{self._globals['tests_dir']}/requirements.txt")
@@ -194,32 +193,57 @@ class ToxConfig(IniConfigurator):
 		``[tox]``.
 		"""
 
-		self._ini["tox"]["envlist"] = [*self["tox_py_versions"], "mypy", "build"]
+		if self["third_party_version_matrix"]:
+			third_party_library = list(self["third_party_version_matrix"].keys())[0]
+			third_party_versions = DelimitedList(self["third_party_version_matrix"][third_party_library])
+			matrix_testenv_string = f"-{third_party_library}{{{third_party_versions:,}}}"
+			tox_envs = [v + matrix_testenv_string for v in self["tox_py_versions"]]
+		else:
+			tox_envs = self["tox_py_versions"]
+
+		self._ini["tox"]["envlist"] = [*tox_envs, "mypy", "build"]
 		self._ini["tox"]["skip_missing_interpreters"] = True
-		self._ini["tox"]["requires"] = indent_join([
-				"pip>=20.3.3",
-				"tox-envlist>=0.1.0",
-				*self["tox_requirements"],
-				])
 		self._ini["tox"]["isolated_build"] = True
 
+		tox_requires = {"pip>=20.3.3", "tox-envlist>=0.2.1", "tox-pip-version>=0.0.7", *self["tox_requirements"]}
+		self._ini["tox"]["requires"] = indent_join(sorted(tox_requires))
+
 	def envlists(self):
 		"""
 		``[envlists]``.
 		"""
 
-		self._ini["envlists"]["test"] = self["tox_py_versions"]
+		if self["third_party_version_matrix"]:
+			third_party_library, third_party_versions, matrix_testenv_string = self.get_third_party_version_matrix()
+			tox_envs = [v + matrix_testenv_string for v in self["tox_py_versions"]]
+			cov_envlist = [
+					f"{self['tox_py_versions'][0]}-{third_party_library}{third_party_versions[0]}",
+					"coverage",
+					]
+		else:
+			tox_envs = self["tox_py_versions"]
+			cov_envlist = [self["tox_py_versions"][0], "coverage"]
+
+		self._ini["envlists"]["test"] = tox_envs
 		self._ini["envlists"]["qa"] = ["mypy", "lint"]
 		if self["enable_tests"]:
-			self._ini["envlists"]["cov"] = [self["tox_py_versions"][0], "coverage"]
+			self._ini["envlists"]["cov"] = cov_envlist
 
-	def gh_actions(self):
+	def get_third_party_version_matrix(self) -> Tuple[str, DelimitedList, str]:
 		"""
-		``[gh-actions]``.
+		Returns information about the matrix of third party versions.
+
+		The returned object is a three-element tuple, comprising:
+
+		* The name of the third party library.
+		* A list of version strings.
+		* The testenv suffix, e.g. ``-attrs{19.3,20.1}``.
 		"""
 
-		versions = (f"{py_ver}: {tox_py_ver}" for py_ver, tox_py_ver in self["gh_actions_versions"].items())
-		self._ini["gh-actions"]["python"] = indent_join(versions)
+		third_party_library = list(self["third_party_version_matrix"].keys())[0]
+		third_party_versions = DelimitedList(self["third_party_version_matrix"][third_party_library])
+		matrix_testenv_string = f"-{third_party_library}{{{third_party_versions:,}}}"
+		return third_party_library, third_party_versions, matrix_testenv_string
 
 	def testenv(self):
 		"""
@@ -230,7 +254,42 @@ class ToxConfig(IniConfigurator):
 			self._ini["testenv"]["setenv"] = "PYTHONDEVMODE = 1"
 
 		if self["enable_tests"]:
-			self._ini["testenv"]["deps"] = f"-r{{toxinidir}}/{self['tests_dir']}/requirements.txt"
+
+			deps = [f"-r{{toxinidir}}/{self['tests_dir']}/requirements.txt"]
+
+			if self["third_party_version_matrix"]:
+				third_party_library = list(self["third_party_version_matrix"].keys())[0]
+
+				for version in self["third_party_version_matrix"][third_party_library]:
+					if version == "latest":
+						deps.append(f"{third_party_library}latest: {third_party_library}")
+					else:
+						v = Version(version)
+
+						if v.is_prerelease:
+							deps.append(f"{third_party_library}{version}: {third_party_library}=={version}")
+						else:
+							deps.append(f"{third_party_library}{version}: {third_party_library}~={version}.0")
+
+			self._ini["testenv"]["deps"] = indent_join(deps)
+
+		elif not self["stubs_package"]:
+			deps = ["importcheck>=0.1.0"]
+
+			if self["third_party_version_matrix"]:
+				third_party_library = list(self["third_party_version_matrix"].keys())[0]
+
+				for version in self["third_party_version_matrix"][third_party_library]:
+					if version == "latest":
+						deps.append(f"{third_party_library}latest: {third_party_library}")
+					else:
+						v = Version(version)
+						if v.is_prerelease:
+							deps.append(f"{third_party_library}{version}: {third_party_library}=={version}")
+						else:
+							deps.append(f"{third_party_library}{version}: {third_party_library}~={version}.0")
+
+			self._ini["testenv"]["deps"] = indent_join(deps)
 
 		if self["tox_testenv_extras"]:
 			self._ini["testenv"]["extras"] = self["tox_testenv_extras"]
@@ -245,7 +304,9 @@ class ToxConfig(IniConfigurator):
 			# testenv_commands.append(
 			# 		f"python -m pytest --cov={{envsitepackagesdir}}/{self['import_name']} -r aR {self['tests_dir']}/ {{posargs}}"
 			# 		)
-			testenv_commands.insert(0, '')
+
+		elif not self["stubs_package"]:
+			testenv_commands.append("python -m importcheck {posargs}")
 
 		self._ini["testenv"]["commands"] = indent_join(testenv_commands)
 
@@ -259,15 +320,19 @@ class ToxConfig(IniConfigurator):
 			self._ini["testenv:docs"]["setenv"] = indent_join(envvars)
 
 			self._ini["testenv:docs"]["basepython"] = "python3.8"
+			self._ini["testenv:docs"]["pip_version"] = "pip>=21"
 			self._ini["testenv:docs"]["changedir"] = f"{{toxinidir}}/{self['docs_dir']}"
 
 			if self["tox_testenv_extras"]:
 				self._ini["testenv:docs"]["extras"] = self["tox_testenv_extras"]
 
-			self._ini["testenv:docs"]["deps"] = indent_join([
-					"-r{toxinidir}/requirements.txt",
-					f"-r{{toxinidir}}/{self['docs_dir']}/requirements.txt",
-					], )
+			self._ini["testenv:docs"]["deps"] = f"-r{{toxinidir}}/{self['docs_dir']}/requirements.txt"
+
+			# self._ini["testenv:docs"]["deps"] = indent_join([
+			# 		"-r{toxinidir}/requirements.txt",
+			# 		f"-r{{toxinidir}}/{self['docs_dir']}/requirements.txt",
+			# 		], )
+
 			self._ini["testenv:docs"]["commands"] = "sphinx-build -M html . ./build {posargs}"
 
 		else:
@@ -281,13 +346,15 @@ class ToxConfig(IniConfigurator):
 		self._ini["testenv:build"]["skip_install"] = True
 		self._ini["testenv:build"]["changedir"] = "{toxinidir}"
 		self._ini["testenv:build"]["deps"] = indent_join([
-				"twine>=3.2.0",
-				"pep517>=0.9.1",
+				# "build[virtualenv]>=0.3.0",
+				"virtualenv",
+				"git+https://github.com/pypa/build",
 				"check-wheel-contents>=0.1.0",
+				"twine>=3.2.0",
 				*self["tox_build_requirements"],
 				])
 		self._ini["testenv:build"]["commands"] = indent_join([
-				'python -m pep517.build --source --binary "{toxinidir}"',
+				'python -m build --sdist --wheel "{toxinidir}"',
 				# python setup.py {posargs} sdist bdist_wheel
 				# "twine check dist/*",
 				"twine check dist/*.tar.gz dist/*.whl",  # source
@@ -300,10 +367,17 @@ class ToxConfig(IniConfigurator):
 		"""
 
 		# TODO: https://github.com/asottile/yesqa
-		self._ini["testenv:lint"]["basepython"] = "python{min_py_version}".format(**self._globals)
+		self._ini["testenv:lint"]["basepython"] = "python{python_deploy_version}".format(**self._globals)
 		self._ini["testenv:lint"]["changedir"] = "{toxinidir}"
 		self._ini["testenv:lint"]["ignore_errors"] = True
-		self._ini["testenv:lint"]["skip_install"] = True
+
+		if self["pypi_name"] in {"domdf_python_tools", "consolekit"}:
+			self._ini["testenv:lint"]["skip_install"] = False
+		elif self["pypi_name"].startswith("flake8"):
+			self._ini["testenv:lint"]["skip_install"] = False
+		else:
+			self._ini["testenv:lint"]["skip_install"] = True
+
 		self._ini["testenv:lint"]["deps"] = indent_join([
 				# "autopep8 >=1.5.2",
 				"flake8 >=3.8.2",
@@ -317,10 +391,12 @@ class ToxConfig(IniConfigurator):
 				"flake8-sphinx-links>=0.0.4",
 				"flake8-strftime>=0.1.1",
 				"flake8-typing-imports>=1.10.0",
+				"flake8-encodings>=0.1.0",
+				"flake8-slots>=0.1.0",
 				"git+https://github.com/PyCQA/pydocstyle@5118faa7173b0e5bbc230c4adf628758e13605bf",
 				"git+https://github.com/domdfcoding/flake8-quotes.git",
 				"git+https://github.com/domdfcoding/flake8-rst-docstrings.git",
-				"git+https://github.com/domdfcoding/flake8-rst-docstrings-sphinx.git@v2",
+				"git+https://github.com/domdfcoding/flake8-rst-docstrings-sphinx.git",
 				"pygments>=2.7.1",
 				])
 		# cmd = f"flake8 {' '.join(self.get_source_files())} --format=rst-toolbox"
@@ -332,23 +408,21 @@ class ToxConfig(IniConfigurator):
 		``[testenv:mypy]``.
 		"""
 
-		if not (self["stubs_package"] and not self["enable_tests"]):
-			self._ini["testenv:mypy"]["basepython"] = "python{python_deploy_version}".format(**self._globals)
-			self._ini["testenv:mypy"]["ignore_errors"] = True
-			self._ini["testenv:mypy"]["changedir"] = "{toxinidir}"
+		self._ini["testenv:mypy"]["basepython"] = "python{python_deploy_version}".format(**self._globals)
+		self._ini["testenv:mypy"]["ignore_errors"] = True
+		self._ini["testenv:mypy"]["changedir"] = "{toxinidir}"
 
-			if self._globals["tox_testenv_extras"]:
-				self._ini["testenv:mypy"]["extras"] = self._globals["tox_testenv_extras"]
+		if self._globals["tox_testenv_extras"]:
+			self._ini["testenv:mypy"]["extras"] = self._globals["tox_testenv_extras"]
 
-			self._ini["testenv:mypy"]["deps"] = indent_join(self.get_mypy_dependencies())
+		self._ini["testenv:mypy"]["deps"] = indent_join(self.get_mypy_dependencies())
 
-			commands = self.get_mypy_commands()
+		commands = self.get_mypy_commands()
 
-			if commands:
-				self._ini["testenv:mypy"]["commands"] = indent_join(commands)
-				return
-
-		self._ini.remove_section("testenv:mypy")
+		if commands:
+			self._ini["testenv:mypy"]["commands"] = indent_join(commands)
+		else:
+			self._ini.remove_section("testenv:mypy")
 
 	def testenv_pyup(self):
 		"""
@@ -381,7 +455,7 @@ class ToxConfig(IniConfigurator):
 
 			coverage_deps = ["coverage>=5"]
 			if self["pypi_name"] != "coverage_pyver_pragma":
-				coverage_deps.append("coverage_pyver_pragma>=0.0.6")
+				coverage_deps.append("coverage_pyver_pragma>=0.2.1")
 
 			self._ini["testenv:coverage"]["deps"] = indent_join(coverage_deps)
 
@@ -398,13 +472,18 @@ class ToxConfig(IniConfigurator):
 		``[flake8]``.
 		"""
 
+		test_ignores = list(code_only_warning)
+		test_ignores.remove("E301")
+		test_ignores.remove("E302")
+		test_ignores.remove("E305")
+
 		self._ini["flake8"]["max-line-length"] = "120"
 		self._ini["flake8"]["select"] = f"{DelimitedList(lint_warn_list + code_only_warning): }"
-		self._ini["flake8"]["exclude"] = ','.join([self["docs_dir"], *standard_flake8_excludes])
+		self._ini["flake8"]["extend-exclude"] = ','.join([self["docs_dir"], *standard_flake8_excludes])
 		self._ini["flake8"]["rst-directives"] = indent_join(sorted(allowed_rst_directives))
 		self._ini["flake8"]["per-file-ignores"] = indent_join([
 				'',
-				f"{self['tests_dir']}/*: {' '.join(str(e) for e in code_only_warning)}",
+				f"{self['tests_dir']}/*: {' '.join(str(e) for e in test_ignores)}",
 				f"*/*.pyi: {' '.join(str(e) for e in code_only_warning)}",
 				])
 		self._ini["flake8"]["pytest-parametrize-names-type"] = "csv"
@@ -434,6 +513,7 @@ class ToxConfig(IniConfigurator):
 		``[coverage:report]``.
 		"""
 
+		self._ini["coverage:report"]["fail_under"] = self["min_coverage"]
 		self._ini["coverage:report"]["exclude_lines"] = indent_join([
 				"raise AssertionError",
 				"raise NotImplementedError",
@@ -490,17 +570,28 @@ class ToxConfig(IniConfigurator):
 		if ini_file.is_file():
 			existing_config = ConfigUpdater()
 			existing_config.read(str(ini_file))
+
 			for section in existing_config.sections_blocks():
 				if section.name not in self.managed_sections:
 					self._ini.add_section(section)
 				elif section.name == "coverage:report" and "omit" in section:
 					self._ini["coverage:report"]["omit"] = section["omit"].value
+				elif section.name == "flake8":
+					self.copy_existing_value(section, "rst-roles")
 
-		# TODO: for tox-isolation
-		# [testenv:{py36,py37,py38,pypy3,py39}]
-		# isolate_dirs =
-		#     {toxinidir}/tests
-		#     tox.ini
+					if "rst-directives" in section:
+						existing_directives = section["rst-directives"].value.splitlines()
+						new_directives = self._ini["flake8"]["rst-directives"].value.splitlines()
+						combined_directives = set(map(str.strip, (*new_directives, *existing_directives)))
+						self._ini["flake8"]["rst-directives"] = indent_join(
+								sorted(filter(bool, combined_directives))
+								)
+
+	# TODO: for tox-isolation
+	# [testenv:{py36,py37,py38,pypy3,py39}]
+	# isolate_dirs =
+	#     {toxinidir}/tests
+	#     tox.ini
 
 
 @management.register("tox")
@@ -536,7 +627,7 @@ def make_yapf(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[st
 
 def make_isort(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[str]:
 	"""
-	Add configuration for ``isort``.
+	Remove the ``isort`` configuration file.
 
 	https://github.com/timothycrosley/isort
 
@@ -544,39 +635,149 @@ def make_isort(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[s
 	:param templates:
 	"""
 
-	# if templates.globals["enable_docs"]:
-	# 	with (repo_path / templates.globals["docs_dir"] / "requirements.txt").open(encoding="UTF-8") as fp:
-	# 		doc_requirements = list(requirements.parse(fp))
-	# else:
-	# 	doc_requirements = []
+	isort_file = PathPlus(repo_path / ".isort.cfg")
+	isort_file.unlink(missing_ok=True)
+	assert not isort_file.is_file()
+	return [isort_file.name]
+
+
+# def make_isort(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[str]:
+# 	"""
+# 	Add configuration for ``isort``.
+#
+# 	https://github.com/timothycrosley/isort
+#
+# 	:param repo_path: Path to the repository root.
+# 	:param templates:
+# 	"""
+#
+# 	isort_config = get_isort_config(repo_path, templates)
+#
+# 	isort_file = PathPlus(repo_path / ".isort.cfg")
+# 	isort = ConfigUpdater()
+#
+# 	if isort_file.is_file():
+# 		isort.read(str(isort_file))
+#
+# 	if "settings" not in isort.sections():
+# 		isort.add_section("settings")
+#
+# 	if "known_third_party" in isort["settings"]:
+# 		known_third_party = set(re.split(r"(\n|,\s*)", isort["settings"]["known_third_party"].value))
+# 	else:
+# 		known_third_party = set()
+#
+# 	known_third_party.update(isort_config["known_third_party"])
+# 	known_third_party.add("github")
+# 	known_third_party.add("requests")
+#
+# 	for key, value in isort_config.items():
+# 		isort["settings"][key] = value
+#
+# 	isort["settings"]["known_third_party"] = sorted(filter(bool, map(str.strip, known_third_party)))
+#
+# 	isort["settings"].pop("float_to_top", None)
+# 	isort["settings"].pop("force_to_top", None)
+#
+# 	isort_file.write_clean(str(isort))
+#
+# 	return [isort_file.name]
+
+
+def make_formate_toml(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[str]:
+	"""
+	Add configuration for ``formate``.
+
+	https://formate.readthedocs.io
+
+	:param repo_path: Path to the repository root.
+	:param templates:
+	"""
+
+	known_third_party = set()
 
 	isort_file = PathPlus(repo_path / ".isort.cfg")
-	isort = ConfigUpdater()
+	formate_file = PathPlus(repo_path / "formate.toml")
 
+	isort_config = get_isort_config(repo_path, templates)
+	known_third_party.update(isort_config["known_third_party"])
+
+	if formate_file.is_file():
+		formate_config = toml.loads(formate_file.read_text())
+	else:
+		formate_config = {}
+
+	# Read the isort config file and get "known_third_party" from there
 	if isort_file.is_file():
+		isort = ConfigUpdater()
 		isort.read(str(isort_file))
 
-	if "settings" not in isort.sections():
-		isort.add_section("settings")
+		if "settings" in isort.sections() and "known_third_party" in isort["settings"]:
+			known_third_party.update(re.split(r"(\n|,\s*)", isort["settings"]["known_third_party"].value))
 
-	isort["settings"]["line_length"] = 115
-	isort["settings"]["force_to_top"] = True
-	isort["settings"]["indent"] = '"\t\t"'  # To match what yapf uses
+	isort_file.unlink(missing_ok=True)
+
+	if "hooks" in formate_config and "isort" in formate_config["hooks"]:
+		if "kwargs" in formate_config["hooks"]["isort"]:
+			known_third_party.update(formate_config["hooks"]["isort"]["kwargs"].get("known_third_party", ()))
+
+			for existing_key, value in formate_config["hooks"]["isort"]["kwargs"].items():
+				if existing_key not in isort_config:
+					isort_config[existing_key] = value
+
+	def normalise_underscore(name: str) -> str:
+		return normalize(name.strip()).replace('-', '_')
+
+	isort_config["known_third_party"] = sorted(set(filter(bool, map(normalise_underscore, known_third_party))))
+
+	hooks = {
+			"dynamic_quotes": 10,
+			"collections-import-rewrite": 20,
+			"yapf": {"priority": 30, "kwargs": {"yapf_style": ".style.yapf"}},
+			"reformat-generics": 40,
+			"isort": {"priority": 50, "kwargs": isort_config},
+			"noqa-reformat": 60,
+			"ellipsis-reformat": 70,
+			"squish_stubs": 80,
+			}
+
+	config = {"indent": '\t', "line_length": 115}
+
+	formate_config["hooks"] = hooks
+	formate_config["config"] = config
+
+	formate_file = PathPlus(repo_path / "formate.toml")
+	formate_file.write_clean(toml.dumps(formate_config, encoder=CustomTomlEncoder(dict)))
+
+	return [formate_file.name, isort_file.name]
+
+
+def get_isort_config(repo_path: pathlib.Path, templates: jinja2.Environment) -> Dict[str, Any]:
+	"""
+	Returns a ``key: value`` mapping of configuration for ``isort``.
+
+	https://github.com/timothycrosley/isort
+
+	:param repo_path: Path to the repository root.
+	:param templates:
+	"""
+
+	isort: Dict[str, Any] = {}
+
+	isort["indent"] = "\t\t"  # To match what yapf uses
 
 	# Undocumented 8th option with the closing bracket indented
-	isort["settings"]["multi_line_output"] = 8
-	isort["settings"]["import_heading_stdlib"] = "stdlib"
-	isort["settings"]["import_heading_thirdparty"] = "3rd party"
-	isort["settings"]["import_heading_firstparty"] = "this package"
-	isort["settings"]["import_heading_localfolder"] = "this package"
-	isort["settings"]["balanced_wrapping"] = False
-	isort["settings"]["lines_between_types"] = 0
-	isort["settings"]["use_parentheses"] = True
-	# isort["settings"]["float_to_top"] = True  # TODO: Doesn't work properly; No imports get sorted or floated to the top
-	isort["settings"]["remove_redundant_aliases"] = True
-	isort["settings"]["default_section"] = "THIRDPARTY"
-	if "float_to_top" in isort["settings"]:
-		del isort["settings"]["float_to_top"]
+	isort["multi_line_output"] = 8
+	isort["import_heading_stdlib"] = "stdlib"
+	isort["import_heading_thirdparty"] = "3rd party"
+	isort["import_heading_firstparty"] = "this package"
+	isort["import_heading_localfolder"] = "this package"
+	isort["balanced_wrapping"] = False
+	isort["lines_between_types"] = 0
+	isort["use_parentheses"] = True
+	# isort["float_to_top"] = True  # TODO: Doesn't work properly; No imports get sorted or floated to the top
+	isort["remove_redundant_aliases"] = True
+	isort["default_section"] = "THIRDPARTY"
 
 	if templates.globals["enable_tests"]:
 		test_requirements = read_requirements(
@@ -588,28 +789,15 @@ def make_isort(repo_path: pathlib.Path, templates: jinja2.Environment) -> List[s
 
 	main_requirements = read_requirements(repo_path / "requirements.txt")[0]
 
-	# TODO: extras
-
-	if "known_third_party" in isort["settings"]:
-		all_requirements = set(re.split(r"\n|,\s*", isort["settings"]["known_third_party"].value))
-	else:
-		all_requirements = set()
-
-	for req in (*test_requirements, *main_requirements):  # *doc_requirements,
-		req.name = normalize(req.name)
-		all_requirements.add(req.name)
-
-	all_requirements = {normalize(r) for r in all_requirements}
+	all_requirements = set(map(normalize, map(attrgetter("name"), (*test_requirements, *main_requirements))))
 	all_requirements.discard(templates.globals["import_name"])
 	all_requirements.discard("iniconfig")
 
-	known_third_party = [req.replace('-', '_') for req in sorted({"github", "requests", *all_requirements})]
-	isort["settings"]["known_third_party"] = known_third_party
-	isort["settings"]["known_first_party"] = templates.globals["import_name"]
+	known_third_party = [req.replace('-', '_') for req in sorted(all_requirements)]
+	isort["known_third_party"] = known_third_party
+	isort["known_first_party"] = templates.globals["import_name"]
 
-	isort_file.write_clean(str(isort))
-
-	return [isort_file.name]
+	return isort
 
 
 class TestsRequirementsManager(RequirementsManager):
@@ -629,9 +817,11 @@ class TestsRequirementsManager(RequirementsManager):
 
 	def compile_target_requirements(self) -> None:
 		if self._globals["pypi_name"] != "coverage_pyver_pragma":
-			self.target_requirements.add(ComparableRequirement("coverage-pyver-pragma>=0.0.6"))
+			self.target_requirements.add(ComparableRequirement("coverage-pyver-pragma>=0.2.1"))
 		if self._globals["pypi_name"] != "domdf_python_tools":
-			self.target_requirements.add(ComparableRequirement("domdf-python-tools[testing]>=1.6.0"))
+			self.target_requirements.add(ComparableRequirement("domdf-python-tools[testing]>=2.0.1"))
+		if self._globals["pypi_name"] != "coincidence":
+			self.target_requirements.add(ComparableRequirement("coincidence>=0.1.0"))
 
 	def merge_requirements(self) -> List[str]:
 		current_requirements, comments, invalid_lines = read_requirements(self.req_file, include_invalid=True)
