@@ -46,7 +46,7 @@ import jinja2
 import pytest  # nodep
 import twine.cli  # nodep
 from apeye.url import URL
-from click.testing import CliRunner, Result
+from consolekit.testing import CliRunner, Result
 from domdf_python_tools.paths import PathPlus, in_directory
 from domdf_python_tools.typing import PathLike
 from dulwich.config import StackedConfig
@@ -57,6 +57,7 @@ from southwark.repo import Repo
 # this package
 import repo_helper.utils
 from repo_helper.build import Builder, build_sdist, build_wheel
+from repo_helper.configuration import get_tox_python_versions
 from repo_helper.files.linting import lint_warn_list
 from repo_helper.templates import template_dir
 from repo_helper.utils import brace
@@ -116,7 +117,10 @@ def demo_environment() -> jinja2.Environment:
 			"stubs_package": false,
 			"managed_message": "This file is managed by 'repo_helper'. Don't edit it directly.",
 			"short_desc": "a short description",
-			"on_pypi": true
+			"on_pypi": true,
+			"docs_fail_on_warning": false,
+			"requires_python": "3.6.1",
+			"third_party_version_matrix": {}
 			}
 
 	plus ``lint_warn_list`` = :py:data:`repo_helper.files.linting.lint_warn_list`.
@@ -149,6 +153,7 @@ def demo_environment() -> jinja2.Environment:
 					enable_conda=True,
 					enable_releases=True,
 					python_deploy_version="3.6",
+					requires_python="3.6.1",
 					min_py_version="3.6",
 					modname="hello-world",
 					repo_name="hello-world",
@@ -173,7 +178,9 @@ def demo_environment() -> jinja2.Environment:
 					managed_message="This file is managed by 'repo_helper'. Don't edit it directly.",
 					short_desc="a short description",
 					on_pypi=True,
+					docs_fail_on_warning=False,
 					brace=brace,
+					third_party_version_matrix={},
 					gh_actions_versions={
 							"3.6": "py36, mypy",
 							"3.7": "py37, build",
@@ -181,6 +188,8 @@ def demo_environment() -> jinja2.Environment:
 					)
 			)
 
+	templates.globals["tox_py_versions"] = get_tox_python_versions(templates.globals["python_versions"])
+
 	return templates
 
 
@@ -287,84 +296,78 @@ def builder_smoke_test(
 	"""
 
 	ret = 0
-
 	target_dir = PathPlus(target_dir)
 
-	try:
-		url = GITHUB_COM / username / repository
+	url = GITHUB_COM / username / repository
 
-		if actions:
-			print(f"::group::{username}_{repository}")
-		else:
-			print("==============================================")
-		print(f"Cloning {url!s} -> {target_dir!s}")
+	if actions:
+		print(f"::group::{username}_{repository}")
+	else:
+		print("==============================================")
+	print(f"Cloning {url!s} -> {target_dir!s}")
 
-		if actions:
-			errstream = BytesIO()
-		else:
-			errstream = default_bytes_err_stream
+	if actions:
+		errstream = BytesIO()
+	else:
+		errstream = default_bytes_err_stream
 
-		clone(str(url), str(target_dir), depth=1, errstream=errstream)
+	clone(str(url), str(target_dir), depth=1, errstream=errstream)
 
-		with in_directory(target_dir):
-			# Run their tests
-			# make_pyproject(target_dir, templates)
-			# print((target_dir / "pyproject.toml").read_text())
-			# test_process = Popen(["python3", "-m", "tox", "-n", "test"])
-			# (output, err) = test_process.communicate()
-			# exit_code = test_process.wait()
-			# ret |= exit_code
+	with in_directory(target_dir):
+		# Run their tests
+		# make_pyproject(target_dir, templates)
+		# print((target_dir / "pyproject.toml").read_text())
+		# test_process = Popen(["python3", "-m", "tox", "-n", "test"])
+		# (output, err) = test_process.communicate()
+		# exit_code = test_process.wait()
+		# ret |= exit_code
 
-			# Test pyp517
-			# make_pyproject(target_dir, templates)
-			# print((target_dir / "pyproject.toml").read_text())
-			# tox_process = Popen(["python3", "-m", "tox", "-e", "build"])
-			# (output, err) = tox_process.communicate()
-			# exit_code = tox_process.wait()
-			# ret |= exit_code
+		# Test pyp517
+		# make_pyproject(target_dir, templates)
+		# print((target_dir / "pyproject.toml").read_text())
+		# tox_process = Popen(["python3", "-m", "tox", "-e", "build"])
+		# (output, err) = tox_process.communicate()
+		# exit_code = tox_process.wait()
+		# ret |= exit_code
 
-			# Test repo_helper.build
-			start_time = time.time()
-			build_wheel(target_dir / "dist")
-			build_sdist(target_dir / "dist")
+		# Test repo_helper.build
+		start_time = time.time()
+		build_wheel(target_dir / "dist")
+		build_sdist(target_dir / "dist")
 
-			if conda:
-				with tempfile.TemporaryDirectory() as tmpdir:
-					builder = Builder(
-							repo_dir=PathPlus.cwd(),
-							build_dir=tmpdir,
-							out_dir=target_dir / "conda_dist",
-							verbose=True,
-							)
-					builder.build_conda()
+		if conda:
+			with tempfile.TemporaryDirectory() as tmpdir:
+				builder = Builder(
+						repo_dir=PathPlus.cwd(),
+						build_dir=tmpdir,
+						out_dir=target_dir / "conda_dist",
+						verbose=True,
+						)
+				builder.build_conda()
 
-			build_time = time.time() - start_time
+		build_time = time.time() - start_time
 
-			sys.stdout.flush()
+		sys.stdout.flush()
 
-			# Twine check
-			print("twine check")
-			ret |= twine.cli.dispatch(["check", os.path.join("dist", '*')])
-			sys.stdout.flush()
+		# Twine check
+		print("twine check")
+		ret |= twine.cli.dispatch(["check", os.path.join("dist", '*')])
+		sys.stdout.flush()
 
-			# check_wheel_contents
-			print("check_wheel_contents")
-			runner = CliRunner()
-			result: Result = runner.invoke(
-					check_wheel_contents.__main__.main,
-					catch_exceptions=False,
-					args=["dist"],
-					)
-			ret |= result.exit_code
-			print(result.stdout, flush=True)
+		# check_wheel_contents
+		print("check_wheel_contents")
+		runner = CliRunner()
+		result: Result = runner.invoke(
+				check_wheel_contents.__main__.main,
+				catch_exceptions=False,
+				args=["dist"],
+				)
+		ret |= result.exit_code
+		print(result.stdout, flush=True)
 
-		if actions:
-			print("::endgroup::")
+	if actions:
+		print("::endgroup::")
 
 	# TODO: create virtualenv and install package in it
 
-	except Exception as e:
-		print(e)
-		ret |= 1
-
 	return ret, build_time
