@@ -30,29 +30,25 @@ import pathlib
 import re
 import textwrap
 from datetime import date, timedelta
-from typing import TYPE_CHECKING, Callable, Dict, Iterable, List, Optional, TypeVar, Union
+from typing import Callable, Dict, Iterable, List, Optional, Union
 
 # 3rd party
 import dulwich.repo
 import isort  # type: ignore
 import isort.settings  # type: ignore
+import toml
 import yapf_isort
 from domdf_python_tools.dates import calc_easter
 from domdf_python_tools.import_tools import discover_entry_points
-from domdf_python_tools.paths import PathPlus
+from domdf_python_tools.paths import PathPlus, sort_paths
 from domdf_python_tools.pretty_print import FancyPrinter
 from domdf_python_tools.stringlist import StringList
 from domdf_python_tools.typing import PathLike
-from dulwich.porcelain import open_repo_closing
 from shippinglabel import normalize
-from southwark import status
+from southwark import open_repo_closing, status
 
 # this package
-from repo_helper.configupdater2 import ConfigUpdater
-
-if TYPE_CHECKING:
-	# this package
-	from repo_helper.core import RepoHelper
+from repo_helper.configupdater2 import ConfigUpdater, Section
 
 __all__ = [
 		"IniConfigurator",
@@ -70,6 +66,7 @@ __all__ = [
 		"commit_changes",
 		"stage_changes",
 		"set_gh_actions_versions",
+		"CustomTomlEncoder",
 		]
 
 #: Under normal circumstances returns :meth:`datetime.date.today`.
@@ -323,6 +320,17 @@ class IniConfigurator:
 		self._output.append(str(self._ini))
 		ini_file.write_lines(self._output)
 
+	def copy_existing_value(self, section: Section, key: str):
+		"""
+		Copy the existing value for ``key``, if present, to the new configuration.
+
+		:param section:
+		:param key:
+		"""
+
+		if key in section:
+			self._ini[section.name][key] = section[key].value
+
 
 def easter_egg() -> None:  # noqa: D102,D103  # pragma: no cover
 	easter = calc_easter(today.year)
@@ -350,37 +358,6 @@ def no_dev_versions(versions: Iterable[str]) -> List[str]:
 	return [v for v in versions if not v.endswith("-dev")]
 
 
-def sort_paths(*paths: PathLike) -> List[PathPlus]:
-	"""
-	Sort the list of paths by directory, then by file.
-
-	:param paths:
-	"""
-
-	directories: Dict[str, List[PathPlus]] = {}
-	local_contents: List[PathPlus] = []
-	files: List[PathPlus] = []
-
-	for obj in [PathPlus(path) for path in paths]:
-		if len(obj.parts) > 1:
-			key = obj.parts[0]
-			if key in directories:
-				directories[key].append(obj)
-			else:
-				directories[key] = [obj]
-		else:
-			local_contents.append(obj)
-
-	# sort directories
-	directories = {directory: directories[directory] for directory in sorted(directories.keys())}
-
-	for directory, contents in directories.items():
-		contents = [path.relative_to(directory) for path in contents]
-		files.extend(PathPlus(directory) / path for path in sort_paths(*contents))
-
-	return files + sorted(local_contents)
-
-
 def stage_changes(
 		repo: Union[PathLike, dulwich.repo.Repo],
 		files: Iterable[PathLike],
@@ -407,9 +384,13 @@ def stage_changes(
 		for filename in files:
 			filename = PathPlus(filename)
 
+			if filename.is_absolute():
+				filename = filename.relative_to(repo.path)
+
 			if filename in unstaged_changes or filename in untracked_files:
 				repo.stage(os.path.normpath(filename))
 				staged_files.append(filename)
+
 			elif (
 					filename in stat.staged["add"] or filename in stat.staged["modify"]
 					or filename in stat.staged["delete"]
@@ -462,11 +443,47 @@ def set_gh_actions_versions(py_versions: Iterable[str]) -> List[str]:
 
 	# Keep in sync with https://github.com/actions/python-versions/releases
 
+	if "3.9-dev" in py_versions:
+		py_versions[py_versions.index("3.9-dev")] = "3.9"
 	if "3.10-dev" in py_versions:
-		py_versions.remove("3.10-dev")
-		py_versions.append("3.10.0-alpha.3")
+		py_versions[py_versions.index("3.10-dev")] = "3.10.0-alpha.6"
 	if "3.10" in py_versions:
-		py_versions.remove("3.10")
-		py_versions.append("3.10.0-alpha.3")
+		py_versions[py_versions.index("3.10")] = "3.10.0-alpha.6"
+	if "pypy3" in py_versions:
+		py_versions[py_versions.index("pypy3")] = "pypy-3.6"
+	if "pypy36" in py_versions:
+		py_versions[py_versions.index("pypy36")] = "pypy-3.6"
+	if "pypy3.6" in py_versions:
+		py_versions[py_versions.index("pypy3.6")] = "pypy-3.6"
+	if "pypy37" in py_versions:
+		py_versions[py_versions.index("pypy37")] = "pypy-3.7"
+	if "pypy3.7" in py_versions:
+		py_versions[py_versions.index("pypy3.7")] = "pypy-3.7"
+	if "rustpython" in py_versions:
+		py_versions.remove("rustpython")
 
 	return py_versions
+
+
+class CustomTomlEncoder(toml.TomlEncoder):
+	"""
+	Customised TOML encoder which wraps long lists onto multiple lines.
+	"""
+
+	max_width: int = 100
+
+	def dump_list(self, v):  # noqa: D102
+		single_line = super().dump_list(v)
+
+		if len(single_line) <= self.max_width:
+			return single_line
+
+		retval = StringList(['['])
+
+		with retval.with_indent("    ", 1):
+			for u in v:
+				retval.append(f"{str(self.dump_value(u))},")
+
+		retval.append(']')
+
+		return str(retval)
